"""
/***************************************************************************
PostGisTools
A QGIS plugin
Create tools using SQL and set it as a new plugin
                             -------------------
begin                : 2009-03-04 
copyright            : (C) 2009 by Mauricio de Paulo
email                : mauricio.dev@gmail.com 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import * 
from PyQt4.QtGui import *
from PyQt4.QtSql import *
from qgis.core import *
import os
from xml.dom import minidom
# Initialize Qt resources from file resources.py
import resources

# Import the code for the dialog
from PostGisToolsDialog import PostGisToolsDialog
from aboutDialog import aboutDialog

class PostGisTools: 
    def __init__(self, iface):
        # Save reference to the QGIS interface
        self.iface = iface
        if os.path.exists(".postgistools.ini"):
            ini=open(".postgistools.ini",'r')
            self.tools=ini.readline()
            ini.close()
            print "Loading tools from ",self.tools
        else:
            self.tools=None
        

    def makeCaller (self,n): return lambda: self.run(n)
    
    def initGui(self):  
        self.actionlist=[] #list of actions
        self.callerlist=[] #list of funcions to call run() with id parameter
        
        #create the set "Choose tools XML" menu button
        self.toolfileAction=QAction(QIcon(":/plugins/postgistools/icon.png"), "Choose tools XML file", self.iface.mainWindow())
        # connect the action to the run method
        QObject.connect(self.toolfileAction, SIGNAL("activated()"),self.setToolsFile ) 
        # Add toolbar button and menu item
        #self.iface.addToolBarIcon(self.toolfileAction)
        self.iface.addPluginToMenu("Postgis Tools",self.toolfileAction)
        
        #create the set "About" file menu button
        self.aboutAction=QAction(QIcon(":/plugins/postgistools/icon.png"), "About PostGisTools", self.iface.mainWindow())
        # connect the action to the run method
        QObject.connect(self.aboutAction, SIGNAL("activated()"),self.about ) 
        # Add toolbar button and menu item
        #self.iface.addToolBarIcon(self.aboutAction)
        self.iface.addPluginToMenu("Postgis Tools",self.aboutAction)
        
        #starting xml file reading
        if self.tools!=None:
            xmlfile=open(self.tools)
            dom=minidom.parse(xmlfile)
            tool=dom.firstChild.firstChild
            
            #loads every tool in the file
            while tool: 
                name= tool.getAttribute("name")
                print "Loading tool",name
                
                # Create action that will start plugin configuration
                self.actionlist.append(QAction(QIcon(":/plugins/postgistools/icon.png"), name, self.iface.mainWindow()))
                
                #create a new funcion that calls run() with the id parameter
                self.callerlist.append(self.makeCaller(len(self.actionlist)-1)) 
                
                # connect the action to the run method
                QObject.connect(self.actionlist[-1], SIGNAL("activated()"), self.callerlist[-1]) 
            
                # Add toolbar button and menu item
                self.iface.addPluginToMenu("Postgis Tools", self.actionlist[-1])
                tool=tool.nextSibling
            xmlfile.close()

    def unload(self):
        # Remove every tool menu item and icon
        print "Trying to unload"
        print dir(self.toolfileAction)
        """for i in range(0,len(self.actionlist)):
            self.iface.removePluginMenu("&PostGis Tools",self.actionlist[i])
            self.iface.removeToolBarIcon(self.actionlist[i])"""
        self.iface.removePluginMenu("PostGis Tools",self.toolfileAction)
        self.iface.removePluginMenu("PostGis Tools",self.aboutAction)
        
    def about(self):
        print "ok"
        self.aboutDlg = aboutDialog() 
        self.aboutDlg.show()# show the dialog
        result = self.aboutDlg.exec_() 

    def runSQL(self, sql, host, dbname, user, password, port):
        #setting up connection
        db = QSqlDatabase.addDatabase("QPSQL");
        db.setHostName(host);
        db.setDatabaseName(dbname);
        db.setUserName(user);
        db.setPassword(password);
        db.setPort(port)
        ok = db.open();
        print "Connected to the database."
        #running the query
        query=QSqlQuery()
        if not query.exec_(sql):
            #writes the error in the textEdit widget (last one)
            self.dlg.ui.widgets[-1].append(query.lastError().text())
            return 0
        rec = query.record()
        #print the resulting table's header
        line="| "
        for i in range(0, rec.count()): 
            line+=str(rec.fieldName(i))
            line+=" | "
        self.dlg.ui.widgets[-1].append("")
        self.dlg.ui.widgets[-1].append("Query results:")
        self.dlg.ui.widgets[-1].append(line)
        
        #print the content
        while (query.next()):
            line="| "
            for i in range(0, rec.count()): 
                line+=str(query.value(i).toString())
                line+=" | "
            self.dlg.ui.widgets[-1].append(line)
    
    def listConnections(self):
        #listing connections from qgis
        settings=QSettings()
        settings.beginGroup("/PostgreSQL/connections")
        keys = settings.childGroups()
        dbs=""
        for key in keys:
            dbs+=str(unicode(key))+';'
        settings.endGroup()
        return dbs[:-1]
    
    def setToolsFile(self):
        filename = QFileDialog.getOpenFileName(None, "PostGisTools Choose Tool file", "", "PostGisTools File (*.xml)");
        if filename!="":
            ini=open(".postgistools.ini",'w')
            ini.write(filename)
            ini.close
        ini=open(".postgistools.ini",'r')
        self.tools=ini.readline()
        ini.close()
        self.initGui()

    
    def start(self):
        #reads the info in the widgets and calls the sql command
        sql=self.sql
        for i,item in enumerate(self.dlg.ui.widgets):
            print i, item
            if type(item)==type(QTextEdit()):
                text=str(item.toPlainText())
            elif type(item)==type(QLineEdit()):
                text=str(item.text())
            elif type(item)==type(QDoubleSpinBox()):
                text=str(item.value())
            elif type(item)==type(QComboBox()):
                text=str(item.currentText())
            else:
                text="Error loading widget."
            sql=sql.replace("["+str(i+1)+"]",text)
        print sql
        #reading connection parameters from QGIS
        settings=QSettings()
        key = u"/PostgreSQL/connections/" + str(self.dlg.ui.widgets[-2].currentText())
        get_value = lambda x: settings.value( key + "/" + x )
        get_value_str = lambda x: unicode(get_value(x).toString())
        host, database, username, password = map(get_value_str, ["host", "database", "username", "password"])
        port = get_value("port").toInt()[0]
        self.runSQL(sql,host, database, username, password, port)

    def getTool(self,toolid):
        """Reads the xml file looking for the tool with toolid and returns it SQL Query and the parameters double list."""
        xmlfile=open(self.tools)
        dom=minidom.parse(xmlfile)
        tool=dom.firstChild.firstChild
        for i in range(0,toolid):
            tool=tool.nextSibling
        query=tool.getAttribute("query")
        name= tool.getAttribute("name")
        lines=[]
        parm=tool.firstChild
        while parm:
            lines.append( [parm.attributes.getNamedItem("label").value,parm.attributes.getNamedItem("type").value,    parm.attributes.getNamedItem("default").value,parm.attributes.getNamedItem("notnull").value])
            parm=parm.nextSibling
        xmlfile.close()
        return name,query,lines
        
    
    # run method that performs all the real work
    def run(self,actionid): 
        #reads the xml file
        name,self.sql,parameters= self.getTool(actionid)
        #adds the db combobox
        parameters.append(["Database:","comboBox",self.listConnections(),"true"])
        parameters.append(["Result:", "textEdit", "Postgis geoprocessing tool started.", "false"])
        # create and show the dialog 
        self.dlg = PostGisToolsDialog(parameters) 
        self.dlg.setWindowTitle(name)
        #connect the slots
        QObject.connect(self.dlg.ui.buttonBox, SIGNAL("accepted()"), self.start)
            
        # show the dialog
        self.dlg.show()
        result = self.dlg.exec_() 
        # See if OK was pressed
        if result == 1: 
            # do something useful (delete the line containing pass and
            # substitute with your code
            print "ok pressed"
        

if __name__=="__main__":
    import sys
    from PyQt4 import QtGui
    app = QtGui.QApplication(sys.argv)
    p=PostGisTools("a")
    p.run()
    sys.exit(app.exec_())
